@@ -26,6 +26,7 @@ module Control.Monad.ST.Trans.Internal where
 import GHC.Base
 import GHC.ST hiding (liftST)
 
+import qualified Control.Monad.Fail as MF
 import Control.Monad.Fix
 import Control.Monad.Trans
 import Control.Monad.Error.Class
@@ -33,12 +34,14 @@ import Control.Monad.Reader.Class
 import Control.Monad.State.Class
 import Control.Monad.Writer.Class
 
+#if __GLASGOW_HASKELL__ <= 708
 import Control.Applicative
+#endif
 
 import Data.Array.ST
 import Data.Array.Base
-import GHC.Int    (Int8,  Int16,  Int32,  Int64)
-import GHC.Word   (Word8, Word16, Word32, Word64)
+import GHC.Int    (      Int8,  Int16,  Int32,  Int64)
+import GHC.Word   (Word, Word8, Word16, Word32, Word64)
 import GHC.Ptr    (Ptr, FunPtr)
 import GHC.Stable (StablePtr)
 
@@ -69,6 +72,8 @@ instance Monad m => Monad (STT s m) where
        case ret of
          STTRet new_st a -> 
              unSTT (k a) new_st
+
+instance MF.MonadFail m => MF.MonadFail (STT s m) where
   fail msg = lift (fail msg)
 
 instance MonadTrans (STT s) where
@@ -127,7 +132,7 @@ instance (Applicative m, Monad m) => MArray (STArray s) e (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -139,7 +144,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) Bool (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -151,7 +156,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) Char (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -163,7 +168,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) Int (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -175,7 +180,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) Word (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -187,7 +192,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) (Ptr a) (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -199,7 +204,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) (FunPtr a) (STT s m) wh
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -211,7 +216,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) Float (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -223,7 +228,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) Double (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -235,7 +240,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) (StablePtr a) (STT s m)
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -247,7 +252,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) Int8 (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -259,7 +264,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) Int16 (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -271,7 +276,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) Int32 (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -283,7 +288,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) Int64 (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -295,7 +300,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) Word8 (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -307,7 +312,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) Word16 (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -319,7 +324,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) Word32 (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
@@ -331,7 +336,7 @@ instance (Applicative m, Monad m) => MArray (STUArray s) Word64 (STT s m) where
     {-# INLINE getNumElements #-}
     getNumElements arr = liftST (getNumElements arr)
     {-# INLINE newArray #-}
-    newArray bounds e = liftST (newArray bounds e)
+    newArray bnds e = liftST (newArray bnds e)
     {-# INLINE unsafeRead #-}
     unsafeRead arr i = liftST (unsafeRead arr i)
     {-# INLINE unsafeWrite #-}
